%%%
%%% Compute hitting time using various methods
%%% see Appendix D.1.2.1 in our book
%%%

clear all
%
% 1-st method: 
% computing hitting time by using Laplacian pseudoinversion
%
a=load('matrix.dat');
n = length(a);
e = ones(n,1);
d = sum(a,2);

L = diag(sum(a,2)) - a; %graph Laplacian

% Matrix L^{+}
% Laplacian pseudoinverse

Lps = inv(L - e*e'/n) + e*e'/n;

% Now compute the hitting time
M = zeros(n,n);

for k=1:n
   for i=1:n
      for j=1:n
         M(k,i) = M(k,i) + (Lps(i,j) - Lps(i,k) - Lps(k,j) ...
         + Lps(k,k))*d(j);
      end;
   end;
end;

%
% 2-nd method (Lovasz)
%

m = nnz(a)/2;
D = diag(1./sqrt(d));

N = D*a*D; % symmetric probability matrix

[u,v] = eig(N);
vv=diag(v); 
me=find(vv==max(vv));
ord = 1:n;
ord(me)=[];

deg = sum(a,2);


%%%
%%% Remark: in H matrix the indices (j,i) are reversed to (i,j)
%%% 

H=zeros(n);
for i=1:n
   for j=1:n
      for k1=1:n-1
         k = ord(k1);
         H(i,j) = H(i,j) + ((u(i,k))^2/deg(i) - ...
            u(i,k)*u(j,k)/sqrt(deg(i)*deg(j)))/(1 - vv(k));
      end;
      H(i,j) = 2*m*H(i,j);
   end;
end;

err1 = norm(M-H,'fro')

%%%
%%% 3-rd method described by M. Brand in his report
%%% "A random walks perspective on maximizing satisfaction and profit"
%%%

s = d/sum(d);
P = inv(diag(d))*a;
A = inv(eye(n) - P - ones(n,1)*s');
B = inv(eye(n) - P + ones(n,1)*s');

H1=[];
for i=1:n
   for j=1:n
      H1(i,j) = (A(i,i) - A(j,i))/s(i);
   end;
end;

H2=[];
for i=1:n
   for j=1:n
      H2(i,j) = (B(i,i) - B(j,i))/s(i);
   end;
end;

err2=sum(sum(abs(H1-H),2))
err3=sum(sum(abs(H1-M),2))

err4=sum(sum(abs(H1-H2),2))
