%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% Two-step procedure
%% Second paragraph in Section 3.3
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [V,err]=TwoSteps(A,B,UU,p);

%input:
% K  - kernel ti be approximated
% UU - true eigenvectors of K
% p  - number of eigenvectors to be returned (p <= m)

if (p > size(A,1)), error('p exceeds m'); end;

% step one
[V LL]=eig(A);
Us=[V;B'*V*pinv(LL)];

% step two: orthogonalization of Us
Z=Us*sqrtm(LL);

[F, Sig]=eig(Z'*Z);

[tmp,ord]=sort(-diag(Sig)); %,'descend');
F=F(:,ord);
Sig=diag(-tmp);
V=Z*F*diag(1./sqrt(diag(Sig)));

%quality of the approximation
err=zeros(p,1);
for i=1:p
  %V(:,i) = V(:,i)/norm(V(:,i));
  if norm(UU(:,i)+V(:,i))<norm(UU(:,i)-V(:,i))
    V(:,i)=-V(:,i);
  end;
  err(i)=norm(V(:,i)-UU(:,i));
end;


