%% Various implementations of Fowlkes' algorithm
%% described in the paper
%% "Spectral Grouping Using the Nystrom Method"
%% IEEE trans. PAMI, vol. 26, no. 2 Feb. 2004, pp. 214-225

clear all; close all;
x=load('data/2moons.dat');

g=x(:,3);
x=x(:,1:2);
k=max(g);
kolor=['r','b','g'];
n=length(g);
b = 1.2;

%construct full kernel matrix for NJW method
W = exp(-sqdist(x', x')/b);

rp=randperm(n);
W=W(rp,rp);

D=diag(1./sqrt(sum(W,2)));
K=D*W*D; 

K1=K + eps*eye(n); %regularization
[tmp,psd1]=chol(K);
[tmp,psd2]=chol(K1);
fprintf('PSD of (a) K: %d, (b) regularized K: %d\n', psd1,psd2);

K=K1;
clear K1;
[UU,tmp]=eig(K);

m=20; %no. of eigenvectors to be approximated
p=10;

A =K(1:m,1:m);
B =K(1:m,m+1:end);

[V1,err1]=OneShot(A,B,UU(:,1:p),p);
[V2,err2]=TwoSteps(A,B,UU(:,1:p),p);

figure, plot(1:p,err1,'bo-','MarkerSize',8);  hold on;
%plot(1:p,err2,'b.-','MarkerSize',8); 
title('Error of approximation'); grid on
xlabel('eigenvector id'); ylabel('error'); hold on

err1a=norm(V1(:,1:p)-UU(:,1:p),'fro')/norm(UU(:,1:p),'fro');
err2a=norm(V2(:,1:p)-UU(:,1:p),'fro')/norm(UU(:,1:p),'fro');
fprintf('||V1 - U||_F: %f\n',err1a);
fprintf('||V2 - U||_F: %f\n',err2a);

%% Fowlkes algorithm

A =W(1:m,1:m);
B =W(1:m,m+1:end);

[V3,err3]=Fowlkes(A,B,UU,p);
%figure, 
plot(1:p,err3,'r*-','MarkerSize',8); %,'MarkerFaceColor','b','LineWidth',2); 
title('Error of approximation'); grid on
xlabel('eigenvector id'); ylabel('error');

err3a=norm(V3(:,1:p)-UU(:,1:p),'fro')/norm(UU(:,1:p),'fro');
fprintf('||V - U||_F: %f\n',err1);
  

