%%%
%%% An application of the algorithm described in
%%% L. Grady: Random walks for image segmentation. IEEE Trans. on Pattern Analysis
%%% and Machine Intelligence, 28(11):17681783, Nov. 2006
%%%

x=load('data\iris.dat');

g=x(:,5);       %group labels
K=max(g);       %no. of groups
data=x(:,1:4);  %data to be processed
m=length(data); 

dist=sqdist(data',data');
sigma=0.3;
NN=5; %liczba sasiadow w tworzonym grafie

A=zeros(m);
B=zeros(m,NN);

for i=1:m
   [tmp,ind]=sort(dist(i,:));
   B(i,:) = ind(2:NN+1);
end;

%adjacency matrix of kNN-graph
for i=1:m
  for j=1:NN
      if ismember(i,B(B(i,j),:))
         A(i,B(i,j))=1; 
      end;
   end;
end

%make connected the graph representad by A 
d=sum(A,2);
ind=find(d==0);
if length(ind)>0
  for j=1:length(ind)
    A(ind(j),:)=exp(-dist(ind(j),:)/sigma^2);
    A(:,ind(j))=A(ind(j),:);
  end;
end;
isSym=norm(A-A','fro')

L=diag(sum(A,2))-A;

Acc=[];
for iter=1:100
  %prepare labeled data
  rp=randperm(50);
  index=[rp(1) rp(2)];
  rp=randperm(50);
  index=[index rp(1)+50 rp(2)+50 rp(3)+50];
  rp=randperm(50);
  index=[index rp(1)+100 rp(2)+100 rp(3)+100 rp(4)+100 rp(5)+100 rp(6)+100];

  nindex=length(index);
  vals=zeros(nindex,K);
  for j=1:nindex
    vals(j,g(index(j)))=1;
  end;

  newVals=dirichletboundary(L,index,vals);

  %assign objects to appropriate groups
  [tmp,g1]=max(newVals,[],2);

  acc=eval_acc_purity(g,g1);
  Acc=[Acc acc];
end;
 
average_quality = mean(Acc)
standard_deviation = std(Acc)
%%%
%%% alternative computations
%%%

antiIndex=1:m;
antiIndex(index)=[];

P=diag(1./sum(A,2))*A;

PU=P(antiIndex,antiIndex);
PUM=P(antiIndex,index);

nv1=pinv(eye(m-length(index)) - PU)*PUM*vals;

newVals2=zeros(m,K); 
newVals2(index,:)=vals;
newVals2(antiIndex,:)=nv1;

err=norm(newVals-newVals2,'fro')

[tmp,g2]=max(newVals2,[],2);

acc2=eval_acc_purity(g,g2)

